package Crypt::AuthEnc;

use strict;
use warnings;
our $VERSION = '0.089';

### not used

1;

=pod

=head1 NAME

Crypt::AuthEnc - [internal only]

=head1 SYNOPSIS

Do not use this module directly.

Use a concrete authenticated-encryption module instead.

=head1 SEE ALSO

=over

=item * L<CryptX>

=item * L<Crypt::AuthEnc::ChaCha20Poly1305>, L<Crypt::AuthEnc::XChaCha20Poly1305>

=item * L<Crypt::AuthEnc::CCM>, L<Crypt::AuthEnc::EAX>, L<Crypt::AuthEnc::GCM>, L<Crypt::AuthEnc::OCB>, L<Crypt::AuthEnc::SIV>

=back

=cut
